class Day
{
public:
    /**
     * Constructor.
     * Construct a day with a given year, month, and day
     * of the Julian/Gregorian calendar. The Julian calendar
     * is used for all days before October 15, 1582.
     * @param y a year != 0
     * @param m a month between 1 and 12
     * @param d a date between 1 and 31
     */
    Day(int y, int m, int d) : year(y), month(m), date(d) {}

    /**
     * Getter.
     * @return the year of this day.
     */
    int get_year()  { return year; }

    /**
     * Getter.
     * @return the month of this day.
     */
    int get_month() { return month; }

    /**
     * Getter.
     * @return the date of this day.
     */
    int get_date() { return date; }

    /**
     * Return the day that is a given number of days away from this day.
     * @param n the number of days, can be negative.
     * @return the day.
     */
    Day *add_days(int n);

    /**
     * Return the number of days between this day and another day.
     * @param other the other day.
     * @return the number of days (> 0 if this day comes before)
     */
    int days_from(Day *other);

private:
    int year;
    int month;
    int date;

    static const int DAYS_PER_MONTH[];

    static const int GREGORIAN_START_YEAR = 1582;
    static const int GREGORIAN_START_MONTH = 10;
    static const int GREGORIAN_START_DATE = 15;
    static const int JULIAN_END_DATE = 4;

    static const int JANUARY  = 1;
    static const int FEBRUARY = 2;
    static const int DECEMBER = 12;

    /**
     * Get the number of days in a given year and month.
     * @param year the year.
     * @param month the month.
     * @return the number of days.
     */
    static int days_per_month(const int year, const int month);

    /**
     * Test if a given year is a leap year.
     * @param year the year.
     * @return true if a leap year, else false.
     */
    static bool is_leap_year(const int year);

    /**
     * Compare this day with another day.
     * @param other the other day.
     * @return a positive number if this day comes after the
     *         other day, a negative number if this day comes
     *         before the other day, and zero if the days are
     *         the same.
     */
    int compare_to(const Day *other) const;

    /**
     * Compute the next day.
     * @return the day following this day.
     */
    Day *next_day() const;

    /*
     * Compute the previous day.
     * @return the day preceding this day.
     */
    Day *previous_day() const;
};
